// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.lang.reflect.Field;
import java.text.Collator;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.dialogs.FilteredItemsSelectionDialog;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * <p>Abstract AbstractFilteredCdmResourceSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public abstract class AbstractFilteredCdmResourceSelectionDialog<T extends ICdmBase> extends
		FilteredItemsSelectionDialog implements IConversationEnabled {

	private final ConversationHolder conversation;

	protected List<UuidAndTitleCache<T>> model;
	private final Set<T> transientCdmObjects = new HashSet<T>();
	private final String settings;

	protected T cdmBaseToBeFiltered;

	/**
	 * <p>Constructor for AbstractFilteredCdmResourceSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 * @param cdmObject a T object.
	 * @param <T> a T object.
	 */
	protected AbstractFilteredCdmResourceSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, String settings, T cdmObject) {
		super(shell, multi);
		setTitle(title);
		setMessage("Use * for wildcard, or ? to see all entries");
		this.settings = settings;

		this.conversation = conversation;

		init();

		initModel();

		String objectTitle = getTitle(cdmObject);
		if (objectTitle != null) {
			setInitialPattern(objectTitle);
		}

		setListLabelProvider(createListLabelProvider());
		setDetailsLabelProvider(createDetailsLabelProvider());

		setSelectionHistory(new ResourceSelectionHistory());
	}

	/**
	 * By default, we are returning the standard list label provider
	 *
	 * Override in subclasses if you want different behavior
	 *
	 * @return
	 */
	protected ILabelProvider createDetailsLabelProvider() {
		return createListLabelProvider();
	}

	/**
	 *
	 * @return
	 */
	protected ILabelProvider createListLabelProvider() {
		return new FilteredCdmResourceLabelProvider();
	}

	/**
	 * Override in subclasses.
	 * Will run before initModel()
	 */
	protected void init() {

	}

	/**
	 * <p>getSelectionFromDialog</p>
	 *
	 * @param dialog a {@link eu.etaxonomy.taxeditor.ui.dialog.selection.AbstractFilteredCdmResourceSelectionDialog} object.
	 * @param <TYPE> a TYPE object.
	 * @return a TYPE object.
	 */
	protected static <TYPE extends CdmBase> TYPE getSelectionFromDialog(AbstractFilteredCdmResourceSelectionDialog<TYPE> dialog) {
		//dialog.setInitialPattern("");
		int result = dialog.open();

		if (result == Window.CANCEL) {
			return null;
		}

		UUID uuid = dialog.getSelectedUuidAndTitleCache().getUuid();
		if(uuid == null){
			return null;
		}
		return dialog.getCdmObjectByUuid(uuid);
	}

	/**
	 * Check if object was created during the life of this dialog. If not,
	 * retrieve it from the CdmStore.
	 *
	 * @param cdmUuid a {@link java.util.UUID} object.
	 * @return a T object.
	 */
	protected T getCdmObjectByUuid(UUID cdmUuid) {
		for (T cdmObject : transientCdmObjects) {
			if (cdmObject.getUuid().equals(cdmUuid)) {
				return cdmObject;
			}
		}
		return getPersistentObject(cdmUuid);
	}

	/**
	 * <p>getPersistentObject</p>
	 *
	 * @param uuid a {@link java.util.UUID} object.
	 * @return a T object.
	 */
	abstract protected T getPersistentObject(UUID uuid);

	/**
	 * @param cdmObject
	 */
//	protected void addObjectToModel(T cdmObject) {
//		model.add(new UuidAndTitleCache(cdmObject.getClass(), cdmObject.getUuid(), getTitle(cdmObject)));
//		transientCdmObjects.add(cdmObject);
//	}

	/**
	 * <p>isObjectTransient</p>
	 *
	 * @param cdmObject a T object.
	 * @return a boolean.
	 */
	protected boolean isObjectTransient(T cdmObject) {
		return (getPersistentObject(cdmObject.getUuid()) == null);
	}

	/**
	 * <p>getTitle</p>
	 *
	 * @param cdmObject a T object.
	 * @return a {@link java.lang.String} object.
	 */
	protected String getTitle(T cdmObject) {
		if(cdmObject == null){
			return "";
		}

		if (cdmObject instanceof IIdentifiableEntity) {
			return ((IIdentifiableEntity) cdmObject).getTitleCache();
		}

		throw new IllegalArgumentException("Generic method only" +
				" supports cdmObject of type IIdentifiableEntity." +
				" Please implement specific method in subclass.");
	}


	/** {@inheritDoc} */
	@Override
	public void refresh() {
		initModel();
		filterExcludedObjects();
		super.refresh();
	}

	/**
	 * <p>initModel</p>
	 */
	abstract protected void initModel();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createFilter()
	 */
	/** {@inheritDoc} */
	@Override
	protected ItemsFilter createFilter() {
		return new ItemsFilter() {

			/**
			 * Always returns false to enforce refiltering even if the pattern is equal
			 */
			@Override
			public boolean equalsFilter(ItemsFilter filter) {
				return false;
			}

			@Override
			public boolean isConsistentItem(Object item) {
				return false;
			}

			@Override
			public boolean matchItem(Object item) {
				String text = null;
				if(item instanceof UuidAndTitleCache){
					text = ((UuidAndTitleCache) item).getTitleCache();
				}else if(item instanceof String){
					text = (String) item;
				}
				return text != null ? matches(text) : false;
			}

		};
	}


	/**
	 * Set the filter input to the Agent's title cache
	 *
	 * @param cdmObject a T object.
	 */
	protected void setPattern(T cdmObject) {
		// FilteredSelection does some very tricky caching to make sure it
		// runs with high performance.
		// This works for most use cases, but we want to change the model while the dialog is open
		// and all the clever caching prevents the content provider from knowing that the model has changed
		// I am aware, that this is a hack, but the FilteredSelectionDialog API does not offer a convenient
		// way to solve the problem.
		try {
			Field lastCompletedFilter = this.getClass().getSuperclass().getSuperclass().getDeclaredField("lastCompletedFilter");
			lastCompletedFilter.setAccessible(true);
			lastCompletedFilter.set(this, null);
		} catch (SecurityException e) {
			MessagingUtils.error(getClass(), e);
		} catch (NoSuchFieldException e) {
			MessagingUtils.error(getClass(), e);
		} catch (IllegalArgumentException e) {
			MessagingUtils.error(getClass(), e);
		} catch (IllegalAccessException e) {
			MessagingUtils.error(getClass(), e);
		}

		// this also is not the nicest way to do it.
		// I am still amazed, that FilteredSelectionDialog does not offer any methods to change its data
		// once it was opened. Am I doing it wrong?
		String pattern = getTitle(cdmObject);
		((Text) getPatternControl()).setText(pattern);
	}

	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#fillContentProvider(org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.AbstractContentProvider, org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.ItemsFilter, org.eclipse.core.runtime.IProgressMonitor)
	*/
	/** {@inheritDoc} */
	@Override
	protected void fillContentProvider(AbstractContentProvider contentProvider,
		ItemsFilter itemsFilter, IProgressMonitor progressMonitor)
		throws CoreException {
		try {
			if(model != null){
				progressMonitor.beginTask("Looking for entities", model.size());
				for(UuidAndTitleCache<T> element : model){
					contentProvider.add(element, itemsFilter);
					if (progressMonitor.isCanceled()) {
						throw new OperationCanceledException();
					}
					progressMonitor.worked(1);
				}
			}else{
				MessagingUtils.warn(getClass(), "Model for Filtered Selection is null:" + this.getClass().getSimpleName());
			}
		}
		finally {
			progressMonitor.done();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getDialogSettings()
	 */
	/** {@inheritDoc} */
	@Override
	protected IDialogSettings getDialogSettings() {
		IDialogSettings settings = TaxeditorStorePlugin.getDefault().getDialogSettings().getSection(getSettings());

		if (settings == null) {
			settings = TaxeditorStorePlugin.getDefault().getDialogSettings().addNewSection(getSettings());
		}
		return settings;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getElementName(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public String getElementName(Object item) {
		return ((UuidAndTitleCache) item).getTitleCache();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getItemsComparator()
	 */
	/** {@inheritDoc} */
	@Override
	protected Comparator getItemsComparator() {
		return new Comparator<UuidAndTitleCache>() {
			@Override
			public int compare(UuidAndTitleCache entity1,
					UuidAndTitleCache entity2) {
				Collator collator = Collator.getInstance();
				return collator.compare(entity1.getTitleCache(), entity2.getTitleCache());
			}
		};
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#validateItem(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	protected IStatus validateItem(Object item) {
		return Status.OK_STATUS;
	}

	/**
	 * <p>getSelectedUuidAndTitleCache</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.common.UuidAndTitleCache} object.
	 */
	protected UuidAndTitleCache getSelectedUuidAndTitleCache() {
		Object[] result = getResult();
		return result[0] == null ? null : (UuidAndTitleCache) result[0];
	}

	/**
	 * <p>Getter for the field <code>settings</code>.</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getSettings()  {
		if(settings == null){
			throw new IllegalStateException("No SETTINGS set.");
		}
		return settings;
	}

	/**
	 *
	 * @author n.hoffmann
	 * @created Oct 19, 2009
	 * @version 1.0
	 */
	private class ResourceSelectionHistory extends SelectionHistory {
	    /*
	    * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#restoreItemFromMemento(org.eclipse.ui.IMemento)
	   	*/
		@Override
		protected Object restoreItemFromMemento(IMemento element) {
			return element.getString("resource"); //$NON-NLS-1$
	  	}
	  	/*
	  	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#storeItemToMemento(java.lang.Object,
	  	 *      org.eclipse.ui.IMemento)
	  	 */
		@Override
		protected void storeItemToMemento(Object item, IMemento element) {
			element.putString("resource", item.toString()); //$NON-NLS-1$
		}
	}

	/**
	 * <p>getNewWizardLinkText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	protected abstract String getNewWizardLinkText();

	/**
	 * <p>getNewEntityWizard</p>
	 * @param parameter
	 * @return a {@link eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard} object.
	 */
	protected abstract AbstractNewEntityWizard getNewEntityWizard(String parameter);

	public class FilteredCdmResourceLabelProvider extends LabelProvider {
		@Override
		public String getText(Object element) {
			if (element == null) {
				return null;
			}
			UuidAndTitleCache uuidAndTitleCache = (UuidAndTitleCache) element;
			String titleCache = uuidAndTitleCache.getTitleCache();
			if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_ID_IN_ENTITY_SELECTION_DIAOLOG)){
			    titleCache += " ["+uuidAndTitleCache.getId()+"]";
			}
            return titleCache;
		}
	};

	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	/** {@inheritDoc} */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		String newWizardLinkText = getNewWizardLinkText();
        if(newWizardLinkText != null){
			Link link = new Link(parent, SWT.NONE);
			link.setText(newWizardLinkText);
			link.addSelectionListener(getNewWizardLinkSelectionListener());
			return link;
		}
		return null;
	}

	protected SelectionListener getNewWizardLinkSelectionListener(){
		return new SelectionAdapter() {

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {

				AbstractNewEntityWizard wizard = getNewEntityWizard(e.text);

				wizard.init(null, null);
				if(wizard.getEntity() != null) {
					WizardDialog dialog = new WizardDialog(getShell(), wizard);
					int status = dialog.open();

					if (status == IStatus.OK) {

						T entity = (T) wizard.getEntity();
						refresh();
						setPattern(entity);
						getConversationHolder().bind();
					}
					//FIXME : Need to make sure this is a stable fix (ticket 3822)
					getConversationHolder().commit();
				}
			}
		};
	}

	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/** {@inheritDoc} */
	@Override
	public void update(CdmDataChangeMap changeEvents) {}

	/**
	 * Don't want to add for example a taxon or synonym to itself
	 * so filter the list to remove the taxon in question
	 * (<code>cdmBaseToBeFiltered</code>)
	 * so it is not available in the filtered list.
	 */
	private void filterExcludedObjects() {
		if (model != null && cdmBaseToBeFiltered != null) {

			UuidAndTitleCache uuidAndTitleCacheToRemove = null;

			for (UuidAndTitleCache uuidAndTitleCache : model){
				if ((cdmBaseToBeFiltered.getUuid()).equals(uuidAndTitleCache.getUuid())) {
					uuidAndTitleCacheToRemove = uuidAndTitleCache;
				}
			}
			model.remove(uuidAndTitleCacheToRemove);
		}
	}
}
