/**
 *
 */
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @version $Id: $
 */
public class TextWithLabelElement extends AbstractCdmFormElement implements ModifyListener, IEnableableFormElement,
        ISelectable {

    protected Text text;
    private Label label;

    private final boolean isMultiLine;

    public static final int MAX_HEIGHT = 0;
    public static final int SINGLE = -1;

    protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, boolean isMultiLine) {
        super(formFactory, parentElement);
        this.isMultiLine = isMultiLine;
    }

    protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            String initialText, Integer textHeight, int style) {
        this(formFactory, parentElement, labelString, initialText, textHeight, null, false, style);
    }

    protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            String initialText, Integer textHeight, boolean isMultiLine, int style) {
        this(formFactory, parentElement, labelString, initialText, textHeight, null, isMultiLine, style);
    }

    protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            String initialText, Integer textHeight, Integer textLimit, int style) {
        this(formFactory, parentElement, labelString, initialText, textHeight, textLimit, false, style);
    }

    protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            String initialText, Integer textHeight, Integer textLimit, boolean isMultiLine, int style) {
        super(formFactory, parentElement);

        this.isMultiLine = isMultiLine;

        initLabel(formFactory, labelString, isMultiLine, getLayoutComposite());

        initText(formFactory, initialText, textHeight, textLimit, isMultiLine, style, getLayoutComposite());
    }

    protected void initText(CdmFormFactory formFactory, String initialText, Integer textHeight, Integer textLimit,
            boolean isMultiLine, int style, Composite layoutComposite) {
        int scrollStyle = textHeight == null ? SWT.NULL : (SWT.V_SCROLL | SWT.MULTI);

        int combinedStyle = style | SWT.BORDER | scrollStyle;

        // SWT.PASSWORD does not work when SWT.WRAP is set.
        if (style != SWT.PASSWORD) {
            combinedStyle = combinedStyle | SWT.WRAP;
        }

        text = formFactory.createText(layoutComposite, "", combinedStyle);
        text.setTextLimit(textLimit!=null?textLimit:Text.LIMIT);

        addControl(text);

        // text.setWO

        if (textHeight == null) {
            text.addKeyListener(new KeyAdapter() {
                @Override
                public void keyPressed(KeyEvent e) {
                    if (e.character == SWT.CR) {
                        // Don't accept carriage returns as input when in single
                        // line mode
                        e.doit = false;
                    } else if (e.character == SWT.TAB) {
                        // traverse is not working for wrapped text widgets so
                        // we reintroduce it here
                        e.doit = false;
                        TextWithLabelElement.this.text.traverse(SWT.TRAVERSE_TAB_NEXT);
                    }
                }
            });
        }

        TableWrapData layoutData;
        if(isMultiLine){
            layoutData = LayoutConstants.FILL_HORIZONTALLY(2, 1);
        }
        else{
            layoutData = LayoutConstants.FILL();
        }
        if (textHeight != null && textHeight > 0) {
            (layoutData).heightHint = textHeight;
        }

        text.setLayoutData(layoutData);

        text.addModifyListener(this);

        setText(initialText);
    }

    protected void initLabel(CdmFormFactory formFactory, String labelString, boolean isMultiLine, Composite layoutComposite) {
        if (labelString != null) {
            label = formFactory.createLabel(layoutComposite, CdmUtils.Nz(labelString), SWT.NULL);
            addControl(label);
            if(isMultiLine){
                label.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            }
            else{
                label.setLayoutData(LayoutConstants.LEFT());
            }
        }
    }

    /**
     * Get the text of this composites text composite
     *
     * @return a {@link java.lang.String} object.
     */
    public String getText() {
        return text.getText();
    }

    /**
     * Set the text of this composites text composite
     *
     * @param string
     *            a {@link java.lang.String} object.
     */
    public void setText(String string) {
        Listener[] listeners = text.getListeners(SWT.Modify);

        for (Listener listener : listeners) {
            text.removeListener(SWT.Modify, listener);
        }

        text.setText(CdmUtils.Nz(string));

        for (Listener listener : listeners) {
            text.addListener(SWT.Modify, listener);
        }
    }

    /** {@inheritDoc} */
    @Override
    public void modifyText(ModifyEvent e) {
        if(e.widget==text && !isMultiLine){
            Text text = (Text) e.widget;
            boolean hasControlCharacters = false;
            String textString = text.getText();
            int stringLength = textString.length();
            for (int i = 0; i < stringLength; i++) {
                if (Character.isISOControl(textString.charAt(i))) {
                    hasControlCharacters = true;
                    break;
                }
            }
            if(hasControlCharacters){
                //remove control character such as line breaks etc.
                setText(text.getText().replaceAll("\\p{C}", new Character((char)9608).toString()));
            }
        }
        firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
    }

    /** {@inheritDoc} */
    @Override
    public void setEnabled(boolean enabled) {
        text.setEnabled(enabled);
        String symbolicName = enabled ? Resources.COLOR_FONT_DEFAULT : Resources.COLOR_TEXT_DISABLED;
        text.setForeground(getColor(symbolicName));
    }

    @Override
    public boolean isEnabled() {
        return text.isEnabled();
    }

    /** {@inheritDoc} */
    @Override
    public void setIrrelevant(boolean irrelevant) {
        String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;

        Color color = getColor(colorId);
        text.setBackground(color);
    }

    /** {@inheritDoc} */
    @Override
    public void setBackground(Color color) {
        if (label != null) {
            label.setBackground(color);
        }
    }

    @Override
    public void setSelected(boolean selected) {
        setBackground(selected ? SELECTED : getPersistentBackground());
    }

    /** {@inheritDoc} */
    @Override
    public void setFocus() {
        text.setFocus();
    }

    public Control getMainControl() {
        return text;
    }

    public void setTextLimit(int limit) {
        text.setTextLimit(limit);
    }

}
