// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * Visualizes an element of type ENTITY in an {@link AbstractEntityCollectionSection}
 *  and links listener functionalities to it.
 *
 *  @param ENTITY the type of the element which is visualized by this class
 *
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 */
public abstract class AbstractEntityCollectionElement<ENTITY> extends
		AbstractCdmFormElement implements IEntityElement<ENTITY>,
		SelectionListener, IConversationEnabled {

	protected ENTITY entity;

	private final Composite container;

	/**
	 * Composite "around" the actual content. Is used for control action like e.g. remove button
	 */
	private final Composite box;

	private Button btnRemove;
    private Button btnChooseEntity;

	private Color backgroundColor;

	public AbstractEntityCollectionElement(CdmFormFactory formFactory,
	        AbstractFormSection section, ENTITY entity,
	        SelectionListener removeListener, Color backgroundColor, int style) {
	    this(formFactory, section, entity, removeListener, false, backgroundColor, style);
	}
	public AbstractEntityCollectionElement(CdmFormFactory formFactory,
			AbstractFormSection section, ENTITY entity, SelectionListener removeListener,
			boolean isChoosableEntity, Color backgroundColor, int style) {
		super(formFactory, (ICdmFormElement) section);

		init();

		formFactory.addPropertyChangeListener(this);

		box = formFactory.createComposite(section.getLayoutComposite());
		box.setBackgroundMode(SWT.INHERIT_DEFAULT);
		addControl(box);

		TableWrapLayout boxLayout = LayoutConstants.LAYOUT(4, false);
		boxLayout.topMargin = 4;
		boxLayout.bottomMargin = 4;
		box.setLayout(boxLayout);

		box.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		container = formFactory.createComposite(box);
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);

		setLayoutComposite(container);

		addControl(container);
		Layout containerLayout = LayoutConstants.LAYOUT(2, false);

		container.setLayout(containerLayout);
		container.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		if(isChoosableEntity){
		    btnChooseEntity = formFactory.createButton(box, null, SWT.PUSH);
		    addControl(btnChooseEntity);
		    btnChooseEntity.setLayoutData(LayoutConstants.RIGHT());
		    btnChooseEntity.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));
		    btnChooseEntity.setToolTipText("Browse");
		    btnChooseEntity.addListener(SWT.Selection, new Listener() {

		        @Override
		        public void handleEvent(Event event) {
		            ENTITY entity = selectFromDialog();
		            if(entity!=null){
		                if(getParentElement() instanceof AbstractEntityCollectionSection){
		                    ((AbstractEntityCollectionSection)getParentElement()).removeElement(getEntity());
		                    setEntity(entity);
		                    ((AbstractEntityCollectionSection)getParentElement()).addElement(entity);
		                    ((AbstractEntityCollectionSection)getParentElement()).firePropertyChangeEvent(getParentElement());
		                }
		            }
		        }
		    });
		}

		if (removeListener != null) {
			btnRemove = formFactory.createButton(box, null, SWT.PUSH);
			addControl(btnRemove);
			btnRemove.setLayoutData(LayoutConstants.RIGHT());
			btnRemove.setImage(ImageResources
					.getImage(ImageResources.TRASH_ICON));
			btnRemove.setToolTipText("Remove");

			btnRemove.addSelectionListener(removeListener);
		}

		createControls(this, style);

		setEntity(entity);
	}

	/**
	 * Init gets executed before any other setup of the section takes place
	 *
	 * Implement this if you want to configure the section
	 */
	public void init() {
		// default implementation is empty
	}

	public abstract void setEntity(ENTITY entity);

	@Override
    public ENTITY getEntity() {
		return entity;
	}

	/**
	 * Sub classes should override to provide the functionality to choose the
	 * entity from existing ones from the data source.<br>
	 * <b>Note:</b> to enable this functionality sub classes have to set
	 * the corresponding flag in the super constructor
	 * @return an existing entity from the data source
	 */
	protected ENTITY selectFromDialog(){
	    return null;
	}

	public abstract void createControls(ICdmFormElement element, int style);

	/**
	 * Mark <code>this</code> element as selected.
	 */
	@Override
    public void setSelected(boolean selected) {

		for (ICdmFormElement element : getElements()) {
			if (element instanceof ISelectable) {
				((ISelectable) element).setSelected(selected);
			}
		}
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();
		if (getElements().contains(eventSource)) {
			handleEvent(eventSource);
		}
	}

	public abstract void handleEvent(Object eventSource);

	/** {@inheritDoc} */
	@Override
    public void setBackground(Color color) {
		backgroundColor = color;
		super.setBackground(color);
		box.setBackground(color);
		container.setBackground(color);
	}

	/**
	 * {@inheritDoc}
	 *
	 * React when selection occurs
	 */
	@Override
    public void widgetSelected(SelectionEvent e) {

	}

	/** {@inheritDoc} */
	@Override
    public void widgetDefaultSelected(SelectionEvent e) {
	}

	/** {@inheritDoc} */
	@Override
	public Composite getLayoutComposite() {
		return container;
	}

	public Color getBackgroundColor() {
		return backgroundColor;
	}

    public Composite getBox() {
        return box;
    }

	@Override
    public ConversationHolder getConversationHolder() {
		if (getParentElement() instanceof IConversationEnabled) {
			return ((IConversationEnabled) getParentElement())
					.getConversationHolder();
		}
		throw new IllegalArgumentException(
				"Parent element should be IConversationEnabled");
	}

	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap changeEvents) {
	}
}
